Package: sparseMatrixStats
Type: Package
Title: Summary Statistics for Rows and Columns of Sparse Matrices
Version: 1.24.0
Authors@R: person("Constantin", "Ahlmann-Eltze", email = "artjom31415@googlemail.com", 
                  role = c("aut", "cre"), comment = c(ORCID = "0000-0002-3762-068X"))
Description: High performance functions for row and column operations
        on sparse matrices. For example: col / rowMeans2, col /
        rowMedians, col / rowVars etc. Currently, the optimizations are
        limited to data in the column sparse format. This package is
        inspired by the matrixStats package by Henrik Bengtsson.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.1
LinkingTo: Rcpp
Imports: Rcpp, Matrix, matrixStats (>= 0.60.0), methods
Depends: MatrixGenerics (>= 1.5.3)
Suggests: testthat (>= 2.1.0), knitr, bench, rmarkdown, BiocStyle
SystemRequirements: C++11
VignetteBuilder: knitr
URL: https://github.com/const-ae/sparseMatrixStats
BugReports: https://github.com/const-ae/sparseMatrixStats/issues
biocViews: Infrastructure, Software, DataRepresentation
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:52:17 UTC
RemoteUrl: https://github.com/bioc/sparseMatrixStats
RemoteRef: RELEASE_3_23
RemoteSha: 5398800600bb32ea869f97f0131716d1f0cc2dcd
NeedsCompilation: yes
Packaged: 2026-05-05 09:07:44 UTC; root
Author: Constantin Ahlmann-Eltze [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3762-068X>)
Maintainer: Constantin Ahlmann-Eltze <artjom31415@googlemail.com>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-05 09:11:12 UTC; unix
