Package: sparseMatrixStats
Type: Package
Title: Summary Statistics for Rows and Columns of Sparse Matrices
Version: 1.10.0
Authors@R: person("Constantin", "Ahlmann-Eltze", email = "artjom31415@googlemail.com", 
                  role = c("aut", "cre"), comment = c(ORCID = "0000-0002-3762-068X"))
Description: High performance functions for row and column operations on sparse matrices.
    For example: col / rowMeans2, col / rowMedians, col / rowVars etc. Currently,
    the optimizations are limited to data in the column sparse format. 
    This package is inspired by the matrixStats package by Henrik Bengtsson.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.1.1
LinkingTo: Rcpp
Imports: Rcpp, Matrix, matrixStats (>= 0.60.0), methods
Depends: MatrixGenerics (>= 1.5.3)
Suggests: testthat (>= 2.1.0), knitr, bench, rmarkdown, BiocStyle
SystemRequirements: C++11
VignetteBuilder: knitr
URL: https://github.com/const-ae/sparseMatrixStats
BugReports: https://github.com/const-ae/sparseMatrixStats/issues
biocViews: Infrastructure, Software, DataRepresentation
git_url: https://git.bioconductor.org/packages/sparseMatrixStats
git_branch: RELEASE_3_16
git_last_commit: 75d85ba
git_last_commit_date: 2022-11-01
Date/Publication: 2022-11-01
NeedsCompilation: yes
Packaged: 2022-11-01 23:04:31 UTC; biocbuild
Author: Constantin Ahlmann-Eltze [aut, cre]
    (<https://orcid.org/0000-0002-3762-068X>)
Maintainer: Constantin Ahlmann-Eltze <artjom31415@googlemail.com>
