Package: sparseMatrixStats
Type: Package
Title: Summary Statistics for Rows and Columns of Sparse Matrices
Version: 1.2.1
Authors@R: person("Constantin", "Ahlmann-Eltze", email = "artjom31415@googlemail.com", 
                  role = c("aut", "cre"), comment = c(ORCID = "0000-0002-3762-068X"))
Description: High performance functions for row and column operations on sparse matrices.
    For example: col / rowMeans2, col / rowMedians, col / rowVars etc. Currently,
    the optimizations are limited to data in the column sparse format. 
    This package is inspired by the matrixStats package by Henrik Bengtsson.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.1.1
LinkingTo: Rcpp
Imports: Rcpp, Matrix, matrixStats, methods
Depends: MatrixGenerics
Suggests: testthat (>= 2.1.0), knitr, bench, rmarkdown, BiocStyle
SystemRequirements: C++14
VignetteBuilder: knitr
biocViews: Infrastructure, Software, DataRepresentation
git_url: https://git.bioconductor.org/packages/sparseMatrixStats
git_branch: RELEASE_3_12
git_last_commit: 9726f3d
git_last_commit_date: 2021-02-02
Date/Publication: 2021-02-02
NeedsCompilation: yes
Packaged: 2021-02-03 03:33:08 UTC; biocbuild
Author: Constantin Ahlmann-Eltze [aut, cre]
    (<https://orcid.org/0000-0002-3762-068X>)
Maintainer: Constantin Ahlmann-Eltze <artjom31415@googlemail.com>
