# Generated by roxygen2: do not edit by hand

exportMethods(colAlls)
exportMethods(colAnyNAs)
exportMethods(colAnys)
exportMethods(colAvgsPerRowSet)
exportMethods(colCollapse)
exportMethods(colCounts)
exportMethods(colCummaxs)
exportMethods(colCummins)
exportMethods(colCumprods)
exportMethods(colCumsums)
exportMethods(colDiffs)
exportMethods(colIQRDiffs)
exportMethods(colIQRs)
exportMethods(colLogSumExps)
exportMethods(colMadDiffs)
exportMethods(colMads)
exportMethods(colMaxs)
exportMethods(colMeans2)
exportMethods(colMedians)
exportMethods(colMins)
exportMethods(colOrderStats)
exportMethods(colProds)
exportMethods(colQuantiles)
exportMethods(colRanges)
exportMethods(colRanks)
exportMethods(colSdDiffs)
exportMethods(colSds)
exportMethods(colSums2)
exportMethods(colTabulates)
exportMethods(colVarDiffs)
exportMethods(colVars)
exportMethods(colWeightedMads)
exportMethods(colWeightedMeans)
exportMethods(colWeightedMedians)
exportMethods(colWeightedSds)
exportMethods(colWeightedVars)
exportMethods(rowAlls)
exportMethods(rowAnyNAs)
exportMethods(rowAnys)
exportMethods(rowAvgsPerColSet)
exportMethods(rowCollapse)
exportMethods(rowCounts)
exportMethods(rowCummaxs)
exportMethods(rowCummins)
exportMethods(rowCumprods)
exportMethods(rowCumsums)
exportMethods(rowDiffs)
exportMethods(rowIQRDiffs)
exportMethods(rowIQRs)
exportMethods(rowLogSumExps)
exportMethods(rowMadDiffs)
exportMethods(rowMads)
exportMethods(rowMaxs)
exportMethods(rowMeans2)
exportMethods(rowMedians)
exportMethods(rowMins)
exportMethods(rowOrderStats)
exportMethods(rowProds)
exportMethods(rowQuantiles)
exportMethods(rowRanges)
exportMethods(rowRanks)
exportMethods(rowSdDiffs)
exportMethods(rowSds)
exportMethods(rowSums2)
exportMethods(rowTabulates)
exportMethods(rowVarDiffs)
exportMethods(rowVars)
exportMethods(rowWeightedMads)
exportMethods(rowWeightedMeans)
exportMethods(rowWeightedMedians)
exportMethods(rowWeightedSds)
exportMethods(rowWeightedVars)
import(MatrixGenerics)
import(methods)
importClassesFrom(Matrix,dgCMatrix)
importClassesFrom(Matrix,lgCMatrix)
importFrom(Matrix,t)
importFrom(Rcpp,sourceCpp)
importFrom(matrixStats,allocArray)
importFrom(stats,setNames)
useDynLib(sparseMatrixStats)
