# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

dgCMatrix_colSums2 <- function(matrix, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_colSums2', PACKAGE = 'sparseMatrixStats', matrix, na_rm)
}

dgCMatrix_colMeans2 <- function(matrix, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_colMeans2', PACKAGE = 'sparseMatrixStats', matrix, na_rm)
}

dgCMatrix_colMedians <- function(matrix, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_colMedians', PACKAGE = 'sparseMatrixStats', matrix, na_rm)
}

dgCMatrix_colVars <- function(matrix, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_colVars', PACKAGE = 'sparseMatrixStats', matrix, na_rm)
}

dgCMatrix_colMads <- function(matrix, na_rm, scale_factor) {
    .Call('_sparseMatrixStats_dgCMatrix_colMads', PACKAGE = 'sparseMatrixStats', matrix, na_rm, scale_factor)
}

dgCMatrix_colMins <- function(matrix, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_colMins', PACKAGE = 'sparseMatrixStats', matrix, na_rm)
}

dgCMatrix_colMaxs <- function(matrix, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_colMaxs', PACKAGE = 'sparseMatrixStats', matrix, na_rm)
}

dgCMatrix_colOrderStats <- function(matrix, which, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_colOrderStats', PACKAGE = 'sparseMatrixStats', matrix, which, na_rm)
}

dgCMatrix_colLogSumExps <- function(matrix, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_colLogSumExps', PACKAGE = 'sparseMatrixStats', matrix, na_rm)
}

dgCMatrix_colProds <- function(matrix, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_colProds', PACKAGE = 'sparseMatrixStats', matrix, na_rm)
}

dgCMatrix_colWeightedMeans <- function(matrix, weights, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_colWeightedMeans', PACKAGE = 'sparseMatrixStats', matrix, weights, na_rm)
}

dgCMatrix_colWeightedVars <- function(matrix, weights, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_colWeightedVars', PACKAGE = 'sparseMatrixStats', matrix, weights, na_rm)
}

dgCMatrix_colCounts <- function(matrix, value, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_colCounts', PACKAGE = 'sparseMatrixStats', matrix, value, na_rm)
}

dgCMatrix_colAnyNAs <- function(matrix) {
    .Call('_sparseMatrixStats_dgCMatrix_colAnyNAs', PACKAGE = 'sparseMatrixStats', matrix)
}

dgCMatrix_colAnys <- function(matrix, value, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_colAnys', PACKAGE = 'sparseMatrixStats', matrix, value, na_rm)
}

dgCMatrix_colAlls <- function(matrix, value, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_colAlls', PACKAGE = 'sparseMatrixStats', matrix, value, na_rm)
}

dgCMatrix_colQuantiles <- function(matrix, probs, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_colQuantiles', PACKAGE = 'sparseMatrixStats', matrix, probs, na_rm)
}

dgCMatrix_colTabulate <- function(matrix, sorted_unique_values) {
    .Call('_sparseMatrixStats_dgCMatrix_colTabulate', PACKAGE = 'sparseMatrixStats', matrix, sorted_unique_values)
}

dgCMatrix_colCumsums <- function(matrix) {
    .Call('_sparseMatrixStats_dgCMatrix_colCumsums', PACKAGE = 'sparseMatrixStats', matrix)
}

dgCMatrix_colCumprods <- function(matrix) {
    .Call('_sparseMatrixStats_dgCMatrix_colCumprods', PACKAGE = 'sparseMatrixStats', matrix)
}

dgCMatrix_colCummins <- function(matrix) {
    .Call('_sparseMatrixStats_dgCMatrix_colCummins', PACKAGE = 'sparseMatrixStats', matrix)
}

dgCMatrix_colCummaxs <- function(matrix) {
    .Call('_sparseMatrixStats_dgCMatrix_colCummaxs', PACKAGE = 'sparseMatrixStats', matrix)
}

dgCMatrix_colRanks_num <- function(matrix, ties_method, na_handling, preserve_shape) {
    .Call('_sparseMatrixStats_dgCMatrix_colRanks_num', PACKAGE = 'sparseMatrixStats', matrix, ties_method, na_handling, preserve_shape)
}

dgCMatrix_colRanks_int <- function(matrix, ties_method, na_handling, preserve_shape) {
    .Call('_sparseMatrixStats_dgCMatrix_colRanks_int', PACKAGE = 'sparseMatrixStats', matrix, ties_method, na_handling, preserve_shape)
}

quantile_sparse <- function(values, number_of_zeros, prob) {
    .Call('_sparseMatrixStats_quantile_sparse', PACKAGE = 'sparseMatrixStats', values, number_of_zeros, prob)
}

dgCMatrix_rowSums2 <- function(matrix, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_rowSums2', PACKAGE = 'sparseMatrixStats', matrix, na_rm)
}

dgCMatrix_rowMeans2 <- function(matrix, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_rowMeans2', PACKAGE = 'sparseMatrixStats', matrix, na_rm)
}

dgCMatrix_rowVars <- function(matrix, na_rm) {
    .Call('_sparseMatrixStats_dgCMatrix_rowVars', PACKAGE = 'sparseMatrixStats', matrix, na_rm)
}

