% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{xgCMatrix-class}
\alias{xgCMatrix-class}
\title{Union of double and logical column-sparse matrices}
\description{
Union of dgCMatrix and lgCMatrix
}
